local L = LibStub("AceLocale-3.0"):GetLocale("SpellReminder")

local function getSpellNameFromInfo(info)
	for k,v in ipairs(info) do
		--SpellReminder:Print(v)
		if v == 'spells' then
			--SpellReminder:Print(info[k])
			return info[k+2]
		end
	end
	return info[#info-1]
end
local function getOption(info)
	local key = info
	if type(key) == 'table' then key = info[#info] end
	return SpellReminder.db.profile[key]
end
local function setOption(info, value)
	local key = info
	if type(key) == 'table' then key = info[#info] end
	SpellReminder.db.profile[key] = value
	SpellReminder:CreateAnchors()
end
local function getSpellProp(info)
	local name = getSpellNameFromInfo(info)
	local key = info[#info]
	return SpellReminder.chardb.char.spellSetup[name][key]
end
local function setSpellProp(info,v)
	local name = getSpellNameFromInfo(info)
	local key = info[#info]
	SpellReminder.chardb.char.spellSetup[name][key] = v
	SpellReminder:RefreshSpellOptions()
end
local function isSpellDisabled(info)
	local name = getSpellNameFromInfo(info)
	local key = info[#info]
	return SpellReminder.chardb.char.spellSetup[name].active == false
end

local defaultSpells = {
	type = "group",
	order = 20,
	name = L["Spell Setup"],
	desc = L["Setup spell reminders."],
	args = {
		disableDetection = {
			type = "toggle",
			order = 103,
			name = L["Disable Detection"],
			desc = L["Disables the detection of new spells."],
			get = getOption,
			set = setOption,
			----passvalue = "disableDetection",
		},
		spacer2 = { type = "header", order = 105, name="" },
		defaultcolor = {
			type = "color",
			order = 107,
			name = L["Default Bar Color"],
			desc = L["Set the default color of the reminder bars."],
			get = function () local c = getOption("defaultBarColor") return c.r,c.g,c.b,c.a end,
			set = function (info, r, g, b, a) setOption("defaultBarColor", {["r"]=r, ["g"]=g, ["b"]=b, ["a"]=a}) SpellReminder:RefreshSpellOptions() end,					
		},
		spacer2 = { type = "header", order = 108, name="" },
		ClearAll = {
			type = "execute",
			order = 110,
			name = L["Clear All Spells"],
			desc = L["Will force SpellReminder to forget all options for ALL spells, including detected spells and pet spells."],
			confirm = true,
			func = function () SpellReminder:ClearSpells(true) end,
		},
--[[		ClearInactive = {
			type = "execute",
			order = 110,
			name = L["Clear Inactive Spells"],
			desc = L["Will force SpellReminder to forget all options for inactive spells."],
			confirm = true,
			func = function () SpellReminder:ClearSpells("inactive") end,
		},]]
--		addAll = {
--			type = "execute",
--			order = 115,
--			name = "Scan Spellbook",
--			desc = "Attempts to scan your spellbook and watch all spells.",
--			func = "InstallAllSpells",
--		},
	},
}

function SpellReminder:RefreshFilterPrioOptions()
	local tbl = self.options.args.barSetup.args.filterPrio.args
	for i,k in ipairs(tbl) do
		tbl[i] = nil
	end
	for idx, fltrName in ipairs(self.db.profile.filterPrio) do
		tbl[fltrName] = {
			type = "execute",
			name = fltrName,
			desc = L["Increase the priority of this filter."],
			width = 'full',
			order = idx+5,
			disabled = idx == 1,
			func = function () self.db.profile.filterPrio[idx] = self.db.profile.filterPrio[idx-1]; self.db.profile.filterPrio[idx-1] = fltrName; self:RefreshFilterPrioOptions() end,
		}
	end
	tbl['_description'] = {
		type = "header",
		name = L["Click on a filter to increase its priority."],
		order = 1,
	}
	tbl['_highest'] = {
		type = "header",
		name = L["Highest Priority"],
		order = 2,
	}
	tbl['_lowest'] = {
		type = "header",
		name = L["Lowest Priority"],
		order = -1,
	}
end

function SpellReminder:RefreshGroupOptions()
	self.options.args.barSetup.args.barGroups.args = { -- setup
			-- add/delete 
		addGroup = {
			type = "input",
			order = 1,
			name = L["Add New"],
			desc = L["Name of the new group."],
			usage = "<name>",
			get = function () return "" end,
			set = function (i,v) self:AddAnchor(v) end
		},
		spacer = { type = "header", order = 2, name="" },
	}
	local pos = 0
	for key,anchor in pairs(self.anchors) do
		self.options.args.barSetup.args.barGroups.args[tostring(key)] = self:GetGroupOptions(key,pos)
		pos = pos + 1
	end
	for key,fltr in pairs(self.options.args.barSetup.args.filters.args) do
		fltr.values = SpellReminder.BarGroupList
	end
end

SpellReminder.BarGroupList = {}
function SpellReminder:RefreshBarGroupList()
	wipe(SpellReminder.BarGroupList)

	SpellReminder.BarGroupList["-1"] = L["None"]
	for k,n in pairs(self.anchors) do
		SpellReminder.BarGroupList[tostring(k)] = n.name or k
	end
end

function SpellReminder:GetGroupOptions(id,pos)
	self:DebugPrint("Getting Group Options: "..id)
	local anchor = self.anchors[id]
	local cantEdit = (id == 'normal' or id == 'emphasis')
	return {
			type = "group",
			name = " "..(anchor.name or id),
			desc = L["Setup Group"],
			order = 50+pos,
			args = {
				anchorName = {
					type = "input",
					order = 1,
					name = L["Group Name"],
					desc = L["Name of the Bar Group"],
					usage = "<name>",
					disabled = cantEdit,
					get = function () return self.db.profile.bargroups[id].name or id end,
					set = function (i,v) self:RenameAnchor(id,v) end,
				},
				scale = {				
					type = "range",
					order = 10,
					name = L["Bar Scale"],
					desc = L["Default Bar Scale."],
					min = 0.2,
					max = 3.0,
					step = 0.1,
					get = function () return self.db.profile.bargroups[id].scale or 1 end,
					set = function (i,v) self.db.profile.bargroups[id].scale = v; self:CreateAnchors() end,
				},
				emphasise = {
					type = "toggle",
					order = 20,
					name = L["Emphasise"],
					desc = L["Bars from this group will emphasise."],
					get = function () return self.db.profile.bargroups[id].emphasise or false end,
					set = function (i,v) self.db.profile.bargroups[id].emphasise = v end,
				},
				growup = {
					type = "toggle",
					order = 30,
					name = L["Grow Upward"],
					desc = L["Bars stack ontop of each other instead of underneath each other."],
					get = function () return self.db.profile.bargroups[id].growup or false end,
					set = function (i,v) self.db.profile.bargroups[id].growup = v; self.anchors[id].LBframe:ReverseGrowth(v); self:CreateAnchors() end,
				},
				reversesort = {
					type = "toggle",
					order = 40,
					name = L["Reverse Sort"],
					desc = L["Sort timer bars in reverse."],
					get = function () return self.db.profile.bargroups[id].reversesort or false end,
					set = function (i,v) self.db.profile.bargroups[id].reversesort = v  self.anchors[id].LBframe:SortBars() end,
				},
				resetPosition = {
					type = "execute",
					name = L["Reset Position"],
					desc = L["Reset group position to center."],
					order = -2,
					confirm = true,
					func = function () self:ResetAnchor(id) end
				},
				deleteGroup = {
					type = "execute",
					name = L["Delete"],
					desc = L["Delete this group"],
					order = -1,
					disabled = cantEdit,
					confirm = true,
					func = function () self:DeleteAnchor(id) end
				},
			},
		}
end

function SpellReminder:RefreshSpellOptions()
	self:DebugPrint("Refreshing Spell Options")
	if self.spellOptions == nil then self.spellOptions = {} end

	for spellName,info in pairs(self.chardb.char.spellSetup) do
		local realSpellName = spellName
		if string.find(realSpellName,":") then
			_,_,_,realSpellName = string.find(realSpellName,"(.*):(.*)")
		end
		if info == nil or info.d == nil or info.d == 0 then
			self.chardb.char.spellSetup[spellName] = nil -- remove invalid spells
			self.spellOptions[spellName] = nil
		elseif (info.last_seen ~= nil and info.last_seen > (time() - (86400*7))) then
			if self.spellOptions[spellName] == nil  then
	--			if self.chardb.char.spellSetup[spellName].showSelf == nil then self.chardb.char.spellSetup[spellName].showSelf = true end
	--			if self.chardb.char.spellSetup[spellName].showOther == nil then self.chardb.char.spellSetup[spellName].showOther = true end
				self.spellOptions[spellName] = {
					type = "group",
					order = 10,
					icon = info.icon,
					name = realSpellName,  -- |cAARRGGBB
					desc = L["Setup reminder for %s|nClick to %s"]:format(spellName,(self.chardb.char.spellSetup[spellName].active and L["deactivate"] or L["activate"])),
					--onClick = function () self.chardb.char.spellSetup[spellName].active = not self.chardb.char.spellSetup[spellName].active SpellReminder:RefreshSpellOptions() SpellReminder:ProcessAuras("player") end,
			--		isChecked = true,
					args = {
	--				purge = {
	--					type = "execute",
	--					order = 1,
	--					name = "Purge",
	--					desc = "Remove this spell",
	--					func = function () self.chardb.char.spellSetup[spellName] = nil SpellReminder:RefreshSpellOptions() end,
	--				},
						active = {
							type = "toggle",
							order = 1,
							name = L['Enabled'],
							get = getSpellProp,
							set = setSpellProp,
						},
						showSelf = {
							type = "toggle",
							order = 1,
							name = L["Show Self"],
							desc = L["Show this spell if its cast on yourself."],
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].selfOnly end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].selfOnly = v end,
							----passvalue = {spellName,"showSelf"},
						},
						showOther = {
							type = "toggle",
							order = 2,
							name = L["Show Others"],
							desc = L["Show this spell when its cast on others"],
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].selfOnly end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].selfOnly = v end,
							--passvalue = {spellName,"showOther"},
						},
						singleTrack = {
							type = "toggle",
							order = 5,
							name = L["Single Tracking"],
							desc = L["This will only track one instance of this spell at any one time.  Useful for Soulstone."],
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
							--passvalue = {spellName,"singleTrack"},
						},
						displayName = {
							type = "input",
							order = 10,
							name = L["Display Name"],
							desc = L["Spell name text to display on the bar. For example an abbreviation."],
							usage = "<name>",
							disabled = isSpellDisabled,
							get = function () return self.chardb.char.spellSetup[spellName].displayName end,
							set = function (i,v) self.chardb.char.spellSetup[spellName].displayName = v end,
							--passvalue = {spellName,"displayName"},
						},
						r = { -- reminder
							type = "range",
							order = 15,
							name = L["Reminder"],
							desc = L["Reminder occurs when the spell duration remaining reaches this value."],
							min = 0,
							max = info.d,
							step = 1,
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].r end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].r = v end,
							--passvalue = {spellName,"r"},
						},
						c = { -- color
							type = "color",
							order = 20,
							name = L["Bar Color"],
							desc = L["Set the color of the reminder bar."],
							disabled = isSpellDisabled,
							get = function (i) local c = self.chardb.char.spellSetup[spellName].c or getOption("defaultBarColor") return c.r,c.g,c.b,c.a end,
							set = function (i, red, green, blue, alpha)
								setSpellProp(i,{["r"]=red, ["g"]=green, ["b"]=blue, ["a"]=alpha})
							end,
							--passvalue = {spellName,"c"},
						},
						sep3 = { type = "header", order = 25, name="" },
		--[[				persistent = {
							type = "toggle",
							order = 12,
							name = "Persistent",
							desc = "If selected, this bar will stay active regardless of death or dispel. Useful for Shadowfiend",
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].persistent == true end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].persistent = v end,
							--passvalue = {spellName,"persistent"},
						},]]
	--[[					fadeTime = {
							type = "range",
							order = 30,
							name = "Fade Time",
							desc = "Time until the bar disapears after it reaches zero. -1 to always show.",
							min = -1,
							max = 10,
							step = 1,
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].fade or 0 end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].fade = v end,
							--passvalue = {spellName,"fade"},
						},]]
						--sep = { type = "header", order = 35, name="" },
						allowCustomSound = {
							type = "toggle",
							order = 40,
							name = L["Allow Custom Sound"],
							desc = L["Allow selection of a custom sound."],
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].selfOnly end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].selfOnly = v end,
							--passvalue = {spellName,"allowCustomSound"},
						},
						customSound = {
							type = "select",
							order = 41,
							name = L["Custom Sound"],
							desc = L["Play a sound when a spell is about to expire."],
							values = SpellReminder.sm:List('sound'),
							disabled = function (t) return isSpellDisabled(t) or (not self.chardb.char.spellSetup[spellName].allowCustomSound) end,
							get = function(t) return self.chardb.char.spellSetup[spellName].customSound or "None" end,
							set = function(t,value) PlaySoundFile(SpellReminder.sm:Fetch('sound',value)) self.chardb.char.spellSetup[spellName].customSound = value end,
							--passvalue = {spellName,"customSound"},
						},
						sep2 = { type = "header", order = 45, name="" },
						allowCustomGroup = {
							type = "toggle",
							order = 50,
							name = L["Allow Custom Group"],
							desc = L["Allow selection of a custom destination group."],
							disabled = isSpellDisabled,
							get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].selfOnly end,
							set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].selfOnly = v end,
							--passvalue = {spellName,"allowCustomGroup"},
						},
						customGroup = {
							type = "select",
							order = 51,
							name = L["Custom Group"],
							desc = L["Which group should this spell be sent to."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							disabled = function (t) return isSpellDisabled(t) or (not self.chardb.char.spellSetup[spellName].allowCustomGroup) end,
							get = getSpellProp,
							set = setSpellProp,
							--passvalue = {spellName,"customGroup"},
						},
						sep2 = { type = "header", order = 55, name="" },
						announceGroup = {
							type = "group",
							order = 56,
							name = L["Announce"],
							desc = L["Setup announce options for %s"]:format(spellName),
							disabled = isSpellDisabled,
							--passvalue = {spellName,"announceGroup"},
							args = {
								announceTo = {
									type = "description",
									order = 1,
									name = L["Announce To"],
								},
								announceSay = {
									type = "toggle",
									order = 1,
									name = L["Say"],
									desc = L["Announce to Say"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceSay"},
								},
								announceYell = {
									type = "toggle",
									order = 2,
									name = L["Yell"],
									desc = L["Announce to Yell"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceYell"},
								},
								announceParty = {
									type = "toggle",
									order = 3,
									name = L["Party"],
									desc = L["Announce to Party"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceParty"},
								},
								announceRaid = {
									type = "toggle",
									order = 4,
									name = L["Raid"],
									desc = L["Announce to Raid"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceRaid"},
								},
								sep = { type = "header", order = 10, name="" },
								announceWhen = {
									type = "description",
									order = 19,
									name = L["Announce When"],
								},
								announceCast = {
									type = "toggle",
									order = 20,
									name = L["Cast"],
									desc = L["Announce when Cast"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceCast"},
								},
								announceEnding = {
									type = "toggle",
									order = 20,
									name = L["Ending"],
									desc = L["Announce when Ending"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceEnding"},
								},
								announceEnded = {
									type = "toggle",
									order = 21,
									name = L["Ended"],
									desc = L["Announce when Ended"],
									get = getSpellProp,--function () return self.chardb.char.spellSetup[spellName].singleTrack end,
									set = setSpellProp,--function (v) self.chardb.char.spellSetup[spellName].singleTrack = v end,
									--passvalue = {spellName,"announceEnded"},
								},
							}
						}
					}
				}
			end
			-- update spell Max duration
				if self.spellOptions[spellName].args.r.max < info.d then
					self.spellOptions[spellName].args.r.max = info.d
				end
			-- end
			-- update spell colour
				if self.chardb.char.spellSetup[spellName].c == nil or type(self.chardb.char.spellSetup[spellName].c.r) ~= "number" then
					self.chardb.char.spellSetup[spellName].c = nil
				end
--setOption('defaultBarColor',{r=0,g=0,b=0})
				local c = self.chardb.char.spellSetup[spellName].c or getOption("defaultBarColor")
				local colorString = string.format("|cFF%.2x%.2x%.2x",c.r*255,c.g*255,c.b*255)
				self.spellOptions[spellName].name = colorString..realSpellName
			-- end
		end
	end
	
	self.options.args.spells = {}
	self:tcopy(defaultSpells,self.options.args.spells)
	for spellName,options in pairs(self.spellOptions) do
		-- get destination
		local destination = self.options.args.spells.args
		if self.chardb.char.spellSetup[spellName].category then
			self.chardb.char.spellSetup[spellName].category = string.gsub(self.chardb.char.spellSetup[spellName].category, "^%s*(.-)%s*$", "%1")
			local cat = self.chardb.char.spellSetup[spellName].category
			if not self.options.args.spells.args[cat] then
				self.options.args.spells.args[cat] = {
					type = "group",
					order = 5,
					name = cat,
					desc = "Options for "..cat,
					args = {},
				}
			end
			destination = self.options.args.spells.args[cat].args
		end
		destination[spellName] = options
	end
end

function SpellReminder:SetSpellCategory(spellName,cat)
	local info = self.chardb.char.spellSetup[spellName]
	if not info or info.category == cat then return end

	info.category = cat
	self:RefreshSpellOptions()
end

SpellReminder.options = {
	type = "group",
	name = "SpellReminder",
	desc = L["Options for SpellReminder."],
	handler = SpellReminder,
	childGroups = "tab",
--	inline = true,
	args = {
		general = {
			type = "group",
			name = "General",
			order = 10,
			args = {
				debugmode = {
					type = "toggle",
					order = 4,
					name = L["Debug Mode"],
					desc = L["Show Debug Messages."],
					get = getOption,
					set = setOption,
					--passvalue = "debugmode"
				},
				minimap = {
					type = "toggle",
					order = 5,
					name = L["Hide Minimap Icon"],
					desc = L["Enable/Disable the Minimap Icon"],
					get = function() return SpellReminder.db.profile.minimap.hide end,
					set = function(i,v)
						SpellReminder.db.profile.minimap.hide = v
						LibStub("LibDBIcon-1.0"):Refresh("SpellReminder")
					end,
				},
				docooldowns = {
					type = "toggle",
					order = 6,
					name = L["Process Cooldowns"],
					desc = L["Creates bars for spell cooldowns."],
					get = getOption,
					set = setOption,
					----passvalue = "docooldowns"
				},
				flashbars = {
					type = "toggle",
					order = 5,
					name = L["Flash Bars"],
					desc = L["Flash bars when they are expiring."],
					get = getOption,
					set = setOption,
					----passvalue = "flashbars"
				},
				spacer4 = { type = "header", order = 6, name="" },
				textWarnings = {
					type = "select",
					name = L["Text Warnings"],
					desc = L["Select which Text Warnings to display"],
					values = { L["Both"] , L["Expiring Only"] , L["Expired Only"] },
					order = 7,
					get = getOption,
					set = setOption,
					----passvalue = 'textWarnings',
				},
				audioWarnings = {
					type = "select",
					name = L["Audiable Warnings"],
					desc = L["Play a sound when a spell is about to expire."],
					values = SpellReminder.sm:List('sound'),
					order = 8,
					get = getOption,
					set = function(_,value) PlaySoundFile(SpellReminder.sm:Fetch('sound',value)) setOption("audioWarnings",value) end,
					----passvalue = 'audioWarnings',
				},
				spacer5 = { type = "header", order = 10, name="" },
			},
		},
		--spells = defaultSpells,
		barSetup = {
			type = "group",
			order = 30,
			name = L["Timer Bars"],
			desc = L["Setup bar animation options and scaling."],
			args = {
			--[[	startScale = {				
					type = "range",
					order = 1,
					name = "Start Scale",
					desc = "Default Bar Scale.",
					min = 0.2,
					max = 2.0,
					step = 0.1,
					get = getOption,
					set = setOption,
					--passvalue = "startScale",
				},
				stopScale = {
					type = "range",
					order = 3,
					name = "Emphasize Scale",
					desc = "Emphasized Bar Scale.",
					disabled = function () return getOption("disableEmphasis") == true end,
					min = 0.2,
					max = 3.0,
					step = 0.1,
					get = getOption,
					set = setOption,
					--passvalue = "stopScale",
				},]]
		--[[		ttg = {
					type = "range",
					order = 4,
					name = "Tween Time",
					desc = "Total time of tween animation.",
					visible = false,
					disabled = function () return getOption("disableEmphasis") == true end,
					min = 0.1,
					max = 3.0,
					step = 0.1,
					get = getOption,
					set = setOption,
					--passvalue = "ttg",
				},
				disableEmphasis = {
					type = "toggle",
					order = 2,
					name = "Disable Emphasis",
					desc = "Disables the emphasise effect",
					get = getOption,
					set = setOption,
					--passvalue = "disableEmphasis",
				},
				spacer2 = { type = "header", order = 5 },]]
				barGroups = {
					type = "group",
					order = 1,
					name = L["Bar Groups"],
					desc = L["Setup Bar Groups"],
					childGroups = "select",
					args = {},
				},
				filters = {
					type = 'group',
					order = 5,
					name = L["Filters"],
					desc = L["Filter different spell types."],
					args = {
						self = {
							type = "select",
							order = 10,
							name = L["Self"],
							desc = L["Which group should self casts be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_self end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_self = v end,
						},
						buffs = {
							type = "select",
							order = 10,
							name = L["Buffs"],
							desc = L["Which group should buffs be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_buff end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_buff = v end,
						},
						debuffs = {
							type = "select",
							order = 10,
							name = L["Debuffs"],
							desc = L["Which group should debuffs be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_debuff end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_debuff = v end,
						},
						cooldowns = {
							type = "select",
							order = 10,
							name = L["Cooldowns"],
							desc = L["Which group should cooldowns be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_cooldown end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_cooldown = v end,
						},
						target = {
							type = "select",
							order = 10,
							name = L["Target"],
							desc = L["Which group should current target be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_target end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_target = v end,
						},
						focus = {
							type = "select",
							order = 10,
							name = L["Focus"],
							desc = L["Which group should focus be sent."],
						--	usage = "<select>",
							values = SpellReminder.BarGroupList,
							get = function () return SpellReminder.db.profile.targetGroup_focus end,
							set = function (i,v) SpellReminder.db.profile.targetGroup_focus = v end,
						},
					},
				},
				filterPrio = {
					type = 'group',
					order = 5,
					name = L["Filter Priorities"],
					desc = L["Set priorities for the filters."],
					args = {},
				},
				spacer1 = { type = "header", order = 10, name="" },
				lockBars = {
					type = "toggle",
					order = 15,
					name = L["Lock timer bars"],
					desc = L["Lock the position of the timer bars."],
					get = function () return getOption("lockbars") end,
					set = function (i,v)
						setOption("lockbars",v)
						-- show/hide the anchors
						SpellReminder:ToggleAnchors()
					end
				},
				resetBars = {
					type = "execute",
					order = 20,
					name = L["Reset Bars"],
					desc = L["Reset the posion of the bars."],
					confirm = true,
					func = "ResetAnchors",
				},
				testBars = {
					type = "execute",
					order = 25,
					name = L["Test Bars"],
					desc = L["Create sample reminder bars to test your layout."],
					func = "CreateTestBars",
				},
				spacer10 = { type = "header", order = 30, name="" },
				formattingSub = {
					type = 'group',
					order = 35,
					name = L["Bar Formatting"],
					desc = L["Various formating options for the bars."],
					args = {
						iconBorder = {
							type = "toggle",
							order = 15,
							name = L["Icon Border"],
							desc = L["Show a border on the icon."],
							get = getOption,
							set = setOption,
							--passvalue = 'iconBorder',
						},
						textFormat = {
							type = "input",
							name = L["Display Text"],
							desc = L["Set the format of the text that appears on the bars.  Possible values are: %n (spell name) %r (spell rank) %t (target) %a (applications)"],
							usage = "<format>",
							order = 51,
							get = getOption,
							set = setOption,
							--passvalue = 'textFormat',
						},
						texture = {
							type = "select",
							name = L["Texture"],
							desc = L["Set the Bar Texture"],
							values = SpellReminder.sm:List('statusbar'),
							order = 52,
							get = getOption,
							set = setOption,
							--passvalue = 'texture',
						},
						spacer3 = { type = "header", order = 100, name="" },
						barwidth = {
							type = 'range',
							name = L["Bar Width"],
							desc = L["Set the Bar Width"],
							order = 110,
							min = 50,
							max = 500,
							step = 10,
							get = getOption,
							set = setOption,
							--passvalue = 'barwidth',
						},
						barheight = {
							type = 'range',
							name = L["Bar Height"],
							desc = L["Set the Bar Height"],
							order = 115,
							min = 5,
							max = 40,
							step = 1,
							get = getOption,
							set = setOption,
							--passvalue = 'barheight',
						},
						spacer4 = { type = "header", order = 120, name="" },
						barfont = {
							type = "select",
							name = L["Font"],
							desc = L["Set the Bar Font"],
							values = SpellReminder.sm:List('font'),
							order = 125,
							get = getOption,
							set = setOption,
							--passvalue = 'barfont',
						},
						barfontsize = {
							type = 'range',
							name = L["Font Size"],
							desc = L["Set the Cast Bar Font Size"],
							order = 130,
							min = 1,
							max = 30,
							step = 1,
							get = getOption,
							set = setOption,
							--passvalue = 'barfontsize',
						},
					},
				},
			},
		},
	},
}